@@ -705,6 +705,7 @@ gdm_greeter_login_window_ready (GdmGreeterLoginWindow *login_window)
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
+        gdk_window_beep (gtk_widget_get_window (GTK_WIDGET (login_window)));
 
         /* If we are retrying a previously selected user */
         if (!login_window->priv->user_list_disabled &&
@@ -1096,6 +1097,7 @@ file_read_one_line (const char *filename)
 
         if (fgets (buf, sizeof (buf), f) == NULL) {
                 g_warning ("Unable to read from file %s", filename);
+                goto out;
         }
 
         line = g_strdup (buf);
@@ -1115,6 +1117,7 @@ static const char *known_etc_info_files [] = {
         "debian_version",
         "mandriva-release",
         "slackware-version",
+        "system-release",
         NULL
 };
 
@@ -1136,6 +1139,14 @@ get_system_version (void)
                         version = g_strdup (output);
                 }
                 version = g_strstrip (version);
+
+                /* lsb_release returns (none) if it doesn't know,
+                 * so return NULL in that case */
+                if (strcmp (version, "(none)") == 0) {
+                        g_free (version);
+                        version = NULL;
+                }
+
                 g_free (output);
 
                 goto out;
